/*	WebDownloader for X-Window
 *	Copyright (C) 1999 Koshelev Maxim
 *	This Program is free but not GPL!!! You can't modify it
 *	without agreement with autor. You can't distribute modified 
 *	program but you can distribute unmodified program.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include "ftpd.h"
#include "ftp.h"
#include "client.h"
#include "liststr.h"
#include "var.h"
#include "locstr.h"
#include "log.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>

//****************************************/
void extract_link(char *src,char *dst) {
    char *delim=" -> ";
    if (src) {
        char *tmp=strstr(src,delim);
        if (tmp) {
            strcpy(dst,tmp+strlen(delim));
            del_crlf(dst);
        } else dst[0]=0;
    };
};

int type_from_str(char *data) {
    if (index(data,'d'))
        return T_DIR;
    if (index(data,'l'))
        return T_LINK;
    if (index(data,'c') ||index(data,'b'))
        return T_DEVICE;
    return T_FILE;
};

int permisions_from_str(char *a) {
    if (a==NULL || strlen(a)<10) return(S_IRUSR|S_IWUSR);
    int perm=0;
    if (a[1]=='r') perm|=S_IRUSR;
    if (a[2]=='w') perm|=S_IWUSR;
    if (a[3]=='x' || a[3]=='s' || a[3]=='S') perm|=S_IXUSR ;
    if (a[4]=='r') perm|=S_IRGRP;
    if (a[5]=='w') perm|=S_IWGRP;
    if (a[6]=='x' || a[6]=='s' || a[6]=='S') perm|=S_IXGRP;
    if (a[7]=='r') perm|=S_IROTH;
    if (a[8]=='w') perm|=S_IWOTH;
    if (a[9]=='x') perm|=S_IXOTH;
    return perm;
};

int date_from_str(char *src){
	char data[MAX_LEN];
	char *tmp=src;
    for (int i=0;i<5;i++) {
        tmp=index(tmp,' ');
        if (tmp) while (*tmp==' ') tmp++;
        else break;
    };
    time_t NOW=time(NULL);
    struct tm date;
    localtime_r(&NOW,&date);
    date.tm_isdst=-1;
    if (tmp && *tmp){
    	tmp=extract_string(tmp,data);
    	date.tm_mon=convert_month(data);
    };
    if (tmp && *tmp){
    	tmp=extract_string(tmp,data);
    	sscanf(data,"%i",&date.tm_mday);
    };
    if (tmp && *tmp){
    	extract_string(tmp,data);
    	if (index(data,':')){
    		sscanf(data,"%i:%i",&date.tm_hour,&date.tm_min);
    	}else{
    		sscanf(data,"%i",&date.tm_year);
    		date.tm_year-=1900;
    	};
    };
  	return mktime(&date);
};

void cut_string_list(char *src,tFileInfo *dst,int flag) {
    char str1[MAX_LEN],name[MAX_LEN],attr[MAX_LEN];
    int par1;
    sscanf(src,"%s %u %s %s %u %s %u %s %s",
           attr,&par1,str1,str1,&dst->size,str1,&par1,str1,name);
    /* Next cycle extract name from string of
     * directory listing
     */
    dst->type=type_from_str(attr);
    char *tmp=src;
    for (int i=0;i<8;i++) {
        tmp=index(tmp,' ');
        if (tmp) while (*tmp==' ') tmp++;
        else break;
    };
    if (flag) {
        if (tmp) dst->name=copy_string(tmp);
        else dst->name=copy_string(name);
        del_crlf(dst->name);
    };
    if (dst->type==T_LINK) {
        extract_link(src,name);
        dst->body=copy_string(name);
        if (flag) {
            tmp=strstr(dst->name," -> ");
            if (tmp) *tmp=0;
        };
    } else dst->body=NULL;
    if (dst->type!=T_DEVICE){
    	    dst->perm=permisions_from_str(attr);
    	    dst->date=date_from_str(src);
    };
};
//****************************************/

int tFtpDownload::change_dir() {
    if (CWDFlag) return 0;
    if (FTP->change_dir(D_PATH)) return -1;
    CWDFlag=1;
    return 0;
};

void tFtpDownload::print_reget(int offset) {
    if (!FTP->test_reget()) {
        LOG->add("Reget not supported!!!\n",LOG_WARNING);
    };
};

void tFtpDownload::check_for_repeated(tStringList *LIST) {
    if (!LIST) return;
    tString *temp,*point=NULL,*first;
    first=LIST->first();
    while((temp=LIST->prev())) {
        if (equal(temp->body,first->body)) {
            point=temp;
            break;
        };
    };
    if (point) {
        temp=LIST->first();
        while(temp && temp!=point) {
            LIST->del(temp);
            delete(temp);
            temp=LIST->first();
        };
    };
};
//****************************************/
tFtpDownload::tFtpDownload() {
    HOST=USER=PASS=D_PATH=NULL;
    D_FILE.name=D_FILE.body=NULL;
    D_FILE.size=D_FILE.type=D_FILE.fdesc=0;
    LOG=NULL;
    FTP=NULL;
    DIR=NULL;
    list=NULL;
    RetrNum=0;
    Status=D_NOTHING;
    config.proxy_host=NULL;
};

int tFtpDownload::reconnect() {
    int success=1;
    Status=D_QUERYING;
    while (success) {
        RetrNum++;
        char data[MAX_LEN];
        if (FTP->get_status()==STATUS_FATAL) return -1;
        if (FTP->get_status()!=STATUS_TIMEOUT && FTP->get_status()!=0) {
            LOG->add("Server refused login probably too many users of your class\n",LOG_WARNING);
        };
        if (config.number_of_attempts)
            sprintf(data,"Retrying %i of %i....\n",RetrNum,config.number_of_attempts);
        else
            sprintf(data,"Retrying %i ...\n",RetrNum);
        LOG->add(data,LOG_OK);
        if (RetrNum==config.number_of_attempts) {
            LOG->add("Max amount of retries was reached!\n",LOG_ERROR);
            return -1;
        };
        FTP->down();
        if (RetrNum>1) {
            if (FTP->test_reget() || CFG.RETRY_IF_NOREGET) {
                LOG->add("Sleeping\n",LOG_OK);
                sleep(config.time_for_sleep);
            } else return -1;
        };
        if (FTP->reinit()==0 && FTP->connect()==0) {
            success=0;
        };
    };
    CWDFlag=0;
    return 0;
};

int tFtpDownload::init(tAddr *hostinfo,tLog *log,tCfg *cfg) {
    LOG=log;
    FTP=new tFtpClient;
    RetrNum=0;
    HOST=hostinfo->host;
    if (hostinfo->username)
        USER=copy_string(hostinfo->username);
    else
        USER=copy_string(DEFAULT_USER);
    if (hostinfo->pass)
        PASS=copy_string(hostinfo->pass);
    else
        PASS=copy_string(DEFAULT_PASS);
    D_PORT=hostinfo->port;
    DIR=NULL;
    list=NULL;
    D_FILE.fdesc=0;
    MASK=hostinfo->mask;
    config.timeout=cfg->timeout;
    config.time_for_sleep=cfg->time_for_sleep;
    config.number_of_attempts=cfg->number_of_attempts;
    config.passive=cfg->passive;
    config.permisions=cfg->permisions;
    config.get_date=cfg->get_date;
    config.proxy_host=copy_string(cfg->proxy_host);
    config.proxy_port=cfg->proxy_port;
    if (config.proxy_host && config.proxy_port){
	    FTP->init(config.proxy_host,LOG,config.proxy_port,config.timeout);
	    char port[MAX_LEN];
	    port[0]=0;
	    if (D_PORT!=21) sprintf(port,":%i",D_PORT);
	    char *temp=new char[strlen(USER)+strlen(config.proxy_host)+strlen("@")+strlen(port)];
	    *temp=0;
	    strcat(temp,USER);
	    strcat(temp,"@");
	    strcat(temp,HOST);
	    if (D_PORT!=21)
		    strcat(temp,port);
		delete USER;
		USER=temp;
    }else
	    FTP->init(HOST,LOG,D_PORT,config.timeout);
   	FTP->registr(USER,PASS);
    FTP->set_passive(config.passive);
    return reconnect();
};

tStringList *tFtpDownload::dir() {
    return DIR;
};

int tFtpDownload::get_size() {
    if (!list) {
        list=new tStringList;
        list->init(0);
    } else list->done();
    int needcheck=0;
    while (1) {
        if (!change_dir()) {
            if (!FTP->stand_data_connection()) {
				int a;
            	if (MASK)
                	a=FTP->get_size(NULL,list);
            	else
                	a=FTP->get_size(D_FILE.name,list);
                if (a==0 && list->count()<=2) {
                    tString *last=list->last();
                    if (!last) {
                        D_FILE.type=T_FILE;
                        D_FILE.size=0;
                        D_FILE.body=NULL;
                        D_FILE.perm=S_IRUSR|S_IWUSR;
                        return 0;
                    };
                    if (equal_first(last->body,"total")) {
                        D_FILE.type=T_DIR;
                        D_FILE.size=1;
                        DIR=list;
                        list=NULL;
                        D_FILE.body=NULL;
                        D_FILE.perm=S_IRUSR|S_IWUSR|S_IXUSR;
                        return 0;
                    };
                    char str1[MAX_LEN];
                    tFileInfo temp;
                    cut_string_list(last->body,&temp,0);
                    D_FILE.type=temp.type;
                    D_FILE.body=temp.body;
                    D_FILE.size=temp.size;
                    D_FILE.perm=temp.perm;
                    D_FILE.date=temp.date;
                    sprintf(str1,"Length is %i\n",D_FILE.size);
                    LOG->add(str1,LOG_OK);
                    return D_FILE.size;
                };
                if (a==0 && list->count()>2) {
                    LOG->add("This is a directory!\n",LOG_WARNING);
                    D_FILE.size=1;
                    if (DIR) delete DIR;
                    DIR=list;
                    list=NULL;
                    D_FILE.type=T_DIR;
                    D_FILE.perm=S_IRUSR|S_IWUSR;
                    if (needcheck) check_for_repeated(DIR);
                    return 1;
                };
                needcheck=1;
                LOG->add("Couldn't get size :((\n",LOG_WARNING);
            } else {
                LOG->add("Can't establish data connection\n",LOG_ERROR);
            };
        } else {
            int s=FTP->get_status();
            if (s!=STATUS_TIMEOUT && s!=STATUS_CMD_ERR)
                break;
            LOG->add("Can't change dir!\n",LOG_ERROR);
        };
        if (reconnect())
            break;
    };
    return -1;
};

int tFtpDownload::download_dir() {
    unsigned int offset=0;
    int ind=0,needcheck=0;
    if (!DIR) {
        DIR=new tStringList;
        DIR->init(0);

        while(1) {
            if (!change_dir()) {
                if (!FTP->stand_data_connection()) {
                    print_reget(offset);
                    if (!FTP->test_reget()) {
                        offset=0;
                        DIR->done();
                    };
                    Status=D_DOWNLOAD;
                    if (MASK)
                    	ind=FTP->get_size(NULL,DIR);
                    else
                    	ind=FTP->get_size(D_FILE.name,DIR);
                    if (ind==0) {
                        LOG->add("Listing was loaded\n",LOG_OK);
                        if (needcheck) check_for_repeated(DIR);
                        return 0;
                    };
                    LOG->add("Listing not loaded completelly\n",LOG_WARNING);
                    needcheck=1;
                } else {
                    LOG->add("Can't establish data connection\n",LOG_ERROR);
                };
            } else {
                LOG->add("Can't change directory\n",LOG_ERROR);
                if (FTP->get_status()==STATUS_FATAL){
                    return -1;
                };
            };
            if (reconnect()){
            	return -1;
            };
        };
    };
    return 0;
};

int tFtpDownload::download(unsigned int from,unsigned int len) {
    unsigned int length=0,offset=from;
    int ind=0;
    if (D_FILE.type==T_DIR) {
        LOG->add("Loading directory...",LOG_OK);
        return(download_dir());
    };
    if (D_FILE.type==T_LINK) {
        LOG->add("Link was loaded :))",LOG_OK);
        return 0;
    };
    while(1) {
        if (!change_dir()) {
            if (!FTP->stand_data_connection()) {
                print_reget(from);
                if (!FTP->test_reget())
                    StartSize=offset=0;
                if (lseek(D_FILE.fdesc,offset,SEEK_SET)<0) {
                    LOG->add("Problems with lseek()",LOG_WARNING);
                };
                Status=D_DOWNLOAD;
                ind=FTP->get_file_from(D_FILE.name,offset,D_FILE.fdesc);
                if (ind>0) {
                    char data[MAX_LEN];
                    length+=ind;
                    offset+=ind;
                    sprintf(data,"%i bytes loaded.",length);
                    LOG->add(data,LOG_OK);
                };
                if (!FTP->get_status()) {
                    if (config.permisions) fchmod(D_FILE.fdesc,D_FILE.perm);
                    return 0;
                };
            } else {
                LOG->add("Can't establish data connection",LOG_ERROR);
            };
        } else {
            LOG->add("Can't change directory",LOG_ERROR);
            if (FTP->get_status()!=STATUS_TIMEOUT){
                return -1;
            };
        };
        if (reconnect()){
        	return -1;
        };
    };
    return 0;
};

void tFtpDownload::set_passive(int a) {
    if (FTP) {
        FTP->set_passive(a);
    };
};

int tFtpDownload::get_readed() {
    return (FTP->get_readed());
};

int tFtpDownload::another_way_get_size(){
	return FTP->another_way_get_size();
};

int tFtpDownload::get_child_status() {
    return(FTP->get_status());
};

int tFtpDownload::get_start_size() {
	if (FTP && !FTP->test_reget())
		return 0;
    return(StartSize);
};

int tFtpDownload::reget(){
	return FTP->test_reget();
};

void tFtpDownload::done() {
    if (FTP) FTP->done();
    if (D_FILE.fdesc) {
        close(D_FILE.fdesc);
        D_FILE.fdesc=0;
    };
};

tFtpDownload::~tFtpDownload() {
    if (FTP) delete(FTP);
    if (D_FILE.fdesc) close(D_FILE.fdesc);
    if (USER) delete(USER);
    if (PASS) delete(PASS);
    if (D_PATH) delete(D_PATH);
    if (D_FILE.name) delete(D_FILE.name);
    if (D_FILE.body) delete D_FILE.body;
    if (DIR) delete(DIR);
    if (list) delete(list);
    if (config.proxy_host) delete(config.proxy_host);
};
