/*	WebDownloader for X-Window
 *	Copyright (C) 1999 Koshelev Maxim
 *	This Program is free but not GPL!!! You can't modify it
 *	without agreement with autor. You can't distribute modified
 *	program but you can distribute unmodified program.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include "hproxy.h"
#include "locstr.h"
#include "var.h"
#include "base64.h"
#include "dlist.h"

tHProxyClient::tHProxyClient(){
	real_host=NULL;
    hostname=userword=username=buffer=NULL;
};

void tHProxyClient::setup_host(char *host){
	real_host=host;
};

int tHProxyClient::get_size(char *filename,tStringList *list){
	char data[MAX_LEN];
    sprintf(data,"GET %s HTTP/1.0\r\n",filename);
    send_request(data);
    sprintf(data,"User-Agent: %s\r\n",VERSION_NAME);
    send_request(data);
    send_request("Accept: */*\r\n");
    sprintf(data,"Range: bytes=%i-\r\n",Offset);
    send_request(data);
    sprintf(data,"Refer: %s\r\n",VERSION_NAME);
    send_request(data);
    sprintf(data,"Host: %s\r\n",real_host);
    send_request(data);
    if (Auth && username && userword) {
        char *tmp=sum_strings(username,":",userword);
        char *pass=string_to_base64(tmp);
        delete tmp;
        sprintf(data,"Proxy-Authorization: Basic %s\r\n",pass);
        delete pass;
        send_request(data);
    };
    send_request("\r\n");
    return read_answer(list);
};

tHProxyClient::~tHProxyClient(){
    if (buffer){
    	delete buffer;
    	buffer=NULL;
    };
    down();
};

/* ---------------------------------------------
 */
tProxyDownload::tProxyDownload(){
    LOG=NULL;
    answer=NULL;
    HTTP=NULL;
    D_FILE.name=HOST=USER=PASS=D_PATH=NULL;
    ETag=Auth=NULL;
    RealName=NewRealName=NULL;
    Status=D_NOTHING;
	config.proxy_host=NULL;
	config.proxy_user=NULL;
	config.proxy_pass=NULL;
	D_PROTO=NULL;
};
 
int tProxyDownload::init(tAddr *hostinfo,tLog *log,tCfg *cfg){
    LOG=log;
    HTTP=new tHProxyClient;
    RetrNum=0;
    HOST=hostinfo->host;
    USER=hostinfo->username;
    PASS=hostinfo->pass;
    D_PORT=hostinfo->port;
    answer=NULL;
    ETag=NULL;
    Auth=NULL;
	D_PATH=NULL;
    D_FILE.name=NULL;
    D_FILE.fdesc=0;
    RealName=NewRealName=NULL;
    data=0;
    first=1;
    config.timeout=cfg->timeout;
    config.time_for_sleep=cfg->time_for_sleep;
    config.number_of_attempts=cfg->number_of_attempts;
    config.proxy_host=copy_string(cfg->proxy_host);
    config.proxy_port=cfg->proxy_port;
    config.proxy_user=copy_string(cfg->proxy_user);
    config.proxy_pass=copy_string(cfg->proxy_pass);
    D_PROTO=copy_string(hostinfo->protocol);
    HTTP->init(config.proxy_host,LOG,config.proxy_port,config.timeout);
    HTTP->registr(config.proxy_user,config.proxy_pass);
    ((tHProxyClient *)(HTTP))->setup_host(HOST);
    return reconnect();
};

int tProxyDownload::get_size() {
// Make a URL from available data
    char fullname[MAX_LEN];
    fullname[0]=0;
    strcat(fullname,D_PROTO);
    strcat(fullname,"://");
    if (USER) {
        strcat(fullname,USER);
        strcat(fullname,":");
        if (PASS) strcat(fullname,PASS);
        strcat(fullname,"@");
    };
    strcat(fullname,HOST);
    char temp[MAX_LEN];
    sprintf(temp,":%i",D_PORT);
    strcat(fullname,temp);
    strcat(fullname,D_PATH);
    if (D_PATH[strlen(D_PATH)-1]!='/')
        strcat(fullname,"/");
    strcat(fullname,D_FILE.name);
//begin request
    if (!answer) {
        answer=new tStringList;
        answer->init(0);
    };
    while (1) {
        answer->done();
        HTTP->set_offset(data);
        LOG->add("Connection to the internet via proxy",LOG_OK);
        LOG->add("Sending request to proxy",LOG_OK);
        if (USER && PASS) HTTP->set_auth(1);
        int temp=HTTP->get_size(fullname,answer);
        if (temp==0) {
            LOG->add("Answer read ok\n",LOG_OK);
            D_FILE.size=analize_answer();
            if (ReGet && D_FILE.size>=0)
                D_FILE.size+=data;
            return D_FILE.size;
        };
        if (temp==1) return -1;
        if (HTTP->get_status()!=STATUS_TIMEOUT) break;
        if (reconnect()) break;
    };
    LOG->add("Could'nt get normal answer!\n",LOG_ERROR);
    return -2;
};

tProxyDownload::~tProxyDownload(){
    if (D_PROTO) delete D_PROTO;
    if (config.proxy_host) delete config.proxy_host;
    if (config.proxy_user) delete config.proxy_user;
    if (config.proxy_pass) delete config.proxy_pass;
/*
    if (HTTP) delete HTTP;
    if (ETag) delete ETag;
    if (Auth) delete Auth;
    if (D_PATH) delete D_PATH;
    if (D_FILE.name) delete (D_FILE.name);
    if (D_FILE.fdesc) close(D_FILE.fdesc);
    if (answer) delete(answer);
    if (RealName) delete RealName;
    if (NewRealName) delete NewRealName;
*/
};
