/*	WebDownloader for X-Window
 *	Copyright (C) 1999 Koshelev Maxim
 *	This Program is free but not GPL!!! You can't modify it
 *	without agreement with autor. You can't distribute modified
 *	program but you can distribute unmodified program.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include "httpd.h"
#include "ftpd.h"
#include "locstr.h"
#include "var.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

tHttpDownload::tHttpDownload() {
    LOG=NULL;
    answer=NULL;
    HTTP=NULL;
    D_FILE.name=HOST=USER=PASS=D_PATH=NULL;
    ETag=Auth=NULL;
    RealName=NULL;
    NewRealName=NULL;
    Auth=NULL;
    Status=D_NOTHING;
    D_FILE.perm=S_IWUSR | S_IRUSR;
};

int tHttpDownload::init(tAddr *hostinfo,tLog *log,tCfg *cfg) {
    LOG=log;
    HTTP=new tHttpClient;
    RetrNum=0;
    HOST=hostinfo->host;
    USER=hostinfo->username;
    PASS=hostinfo->pass;
    D_PORT=hostinfo->port;
    answer=NULL;
    ETag=NULL;
    Auth=NULL;
    D_FILE.name=NULL;
    D_FILE.fdesc=0;
    RealName=NewRealName=NULL;
    data=0;
    first=1;
    config.timeout=cfg->timeout;
    config.time_for_sleep=cfg->time_for_sleep;
    config.number_of_attempts=cfg->number_of_attempts;
    config.get_date=cfg->get_date;
    HTTP->init(HOST,LOG,D_PORT,config.timeout);
    HTTP->registr(USER,PASS);
    return reconnect();
};

void tHttpDownload::init_download(char *path,char *file) {
    tDownloader::init_download(path,file);
    NewRealName=NULL;
    if (RealName) delete RealName;
    RealName=NULL;
    RealName=parse_percents(D_FILE.name);
};

int tHttpDownload::reconnect() {
    int success=1;
    Status=D_QUERYING;
    while (success) {
        RetrNum++;
        char data[MAX_LEN];
        if (HTTP->get_status()==STATUS_FATAL) return -1;
        if (config.number_of_attempts)
            sprintf(data,"Retrying %i of %i....\n",RetrNum,config.number_of_attempts);
        else
            sprintf(data,"Retrying %i\n",RetrNum);
        LOG->add(data,LOG_OK);
        if (RetrNum==config.number_of_attempts) {
            LOG->add("Max amount of retries was reached!\n",LOG_ERROR);
            return -1;
        };
        HTTP->down();
        if (RetrNum>1) {
            if (HTTP->test_reget() || CFG.RETRY_IF_NOREGET) {
                LOG->add("Sleeping\n",LOG_WARNING);
                sleep(config.time_for_sleep);
            } else return -1;
        };
        if (HTTP->reinit()==0)
            success=0;
    };
    return 0;
};

char * tHttpDownload::get_new_url() {
    tString *temp=answer->last();
    char *LOCATION="Location:";
    while(temp) {
        if (equal_first(LOCATION,temp->body)) {
        	int len=strlen(LOCATION);
        	char *tmp=new char[strlen(temp->body)-len+1];
        	char *tmp2=temp->body+len;
        	while (*tmp2 && *tmp2==' ') tmp2+=1;
        	strcpy(tmp,tmp2);
        	del_crlf(tmp);
        	return tmp;
        };
        temp=answer->next();
    };
    return NULL;
};

int tHttpDownload::analize_answer() {
    /*  Here we need few analisation of http answer
     *	AcceptRanges: bytes
     *	Content-Length: (int)
     *	Content-Range: bytes (int)-(int)/(int)
     *	Last-Modified: 
     */
    tString *temp=answer->last();
    if (!temp) return -1;
    char *CL="content-length:";
    char *AR="accept-range:";
    char *CR="content-range:";
    char *CD="content-disposition:";
    char *ET="etag:";
    char *WA="www-authenticate:";
    char *LM="last-modified:";
    int rvalue=0;
    ReGet=0;
    MustNoReget=0;
    while(temp) {
        string_to_low(temp->body,':');
        if (equal_first(AR,temp->body)) {
            if (strstr(temp->body+strlen(AR),"bytes"))
                Status=0;
        };
        if (equal_first(CL,temp->body)) {
            sscanf(temp->body+strlen(CL),"%i",&rvalue);
            char data[MAX_LEN];
            sprintf(data,"Size for download is %i bytes",rvalue);
            LOG->add(data,LOG_OK);
        };
        if (equal_first(CR,temp->body)) {
            if (strstr(temp->body+strlen(CR),"bytes"))
                ReGet=1;
        };
        if (equal_first(LM,temp->body)) {
        	/* Need to extract date
        	 */
        	char *tmp=temp->body+strlen(LM);
        	while (*tmp==' ') tmp++;
			D_FILE.date=ctime_to_time(tmp);
        };
        if (equal_first(ET,temp->body)) {
            char *ETag1=new char[strlen(temp->body)-strlen(ET)];
            sscanf(temp->body+strlen(ET)+1,"%s",ETag1);
            char *tmp=index(ETag1,':');
            if (tmp) *tmp=0;
            if (ETag) {
                if (!equal(ETag,ETag1)) {
                    MustNoReget=1;
                    delete ETag;
                    ETag=ETag1;
                } else delete(ETag1);
            } else {
                ETag=ETag1;
            };
        };
        if (equal_first(CD,temp->body)) {
            char *point=strstr(temp->body,"filename");
            if (point) {
                point+=strlen("filename");
                point=index(point,'=');
                if (point) {
                    point++;
                    NewRealName=copy_string(point);
                    int len=strlen(NewRealName);
                    NewRealName[len-2]=0;
                    point=rindex(NewRealName,'/');
                    if (point){
                    	char *tmp=copy_string(point+1);
                    	delete(NewRealName);
                    	NewRealName=tmp;
                    };
                };
            };
        };
        if (equal_first(WA,temp->body)) {
            if (!Auth) {
                Auth=copy_string(temp->body + strlen(WA)+1);
                Auth[strlen(Auth)-2]=0;
            };
        };
        temp=answer->next();
    };
    if (MustNoReget) ReGet=0;
    return rvalue;
};

int tHttpDownload::get_size() {
    char fullname[MAX_LEN];
    fullname[0]=0;
    strcat(fullname,D_PATH);
    strcat(fullname,"/");
    strcat(fullname,D_FILE.name);
    if (!answer) {
        answer=new tStringList;
        answer->init(0);
    };
    while (1) {
        answer->done();
        HTTP->set_offset(data);
        LOG->add("Sending http request...\n",LOG_OK);
        if (USER && PASS) HTTP->set_auth(1);
        int temp=HTTP->get_size(fullname,answer);
        if (temp==0) {
            LOG->add("Answer read ok\n",LOG_OK);
            D_FILE.size=analize_answer();
            if (ReGet && D_FILE.size>=0)
                D_FILE.size+=data;
            return D_FILE.size;
        };
        if (temp==1) return -1;
        if (HTTP->get_status()!=STATUS_TIMEOUT) break;
        if (reconnect()) break;
    };
    LOG->add("Could'nt get normal answer!\n",LOG_ERROR);
    return -2;
};

int tHttpDownload::download(unsigned int from,unsigned int len) {
    int success=1;
    int offset=from;
    first=1;
    while(success) {
        HTTP->set_offset(offset);
        if (first || get_size()>=0) {
            if (!ReGet) {
                if (offset) LOG->add("It is seemed REGET not supported! Loading from begin..",LOG_WARNING);
                StartSize=data=offset=0;
            };
            if (lseek(D_FILE.fdesc,offset,SEEK_SET)<0) {
                LOG->add("Problems with lseek()\n",LOG_ERROR);
            };
            Status=D_DOWNLOAD;
            int ind=HTTP->get_file_from(NULL,offset,len,D_FILE.fdesc);
            if (ind>=0) {
                offset+=ind;
                len-=ind;
                if (HTTP->get_status()==0){
                	if (D_FILE.size!=0 && offset==D_FILE.size) return 0;
                	if (D_FILE.size==0) return 0;
                };
            };
        };
        data=offset;
        first=0;
        if (HTTP->get_status()==STATUS_FATAL) return -1;
        if (offset==D_FILE.size && D_FILE.size!=0) return 0;
        if (reconnect()) return -1;
    };
    return 0;
};

int tHttpDownload::get_child_status() {
    return HTTP->get_status();
};

int tHttpDownload::get_readed() {
    return HTTP->get_readed();
};

char *tHttpDownload::get_real_name() {
	if (NewRealName) return NewRealName;
    if (RealName)   return RealName;
    return D_FILE.name;
};

int tHttpDownload::reget(){
	if (StartSize) return ReGet;
	return 1;
};

int tHttpDownload::create_file(char *data) {
    if (NewRealName) {
        if (RealName) delete RealName;
        RealName=NewRealName;
        NewRealName=NULL;
    };
    char *temp=D_FILE.name;
    if (RealName) D_FILE.name=RealName;
    D_FILE.type=T_FILE;
    int rvalue=tDownloader::create_file(data);
    if (RealName) D_FILE.name=temp;
    return rvalue;
};


int tHttpDownload::delete_file(char *data) {
    char *temp=D_FILE.name;
    if (RealName) D_FILE.name=RealName;
    int rvalue=tDownloader::delete_file(data);
    if (RealName) D_FILE.name=temp;
    return rvalue;
};


void tHttpDownload::done() {
    HTTP->done();
    if (D_FILE.fdesc) {
        close(D_FILE.fdesc);
        D_FILE.fdesc=0;
    };
};

tHttpDownload::~tHttpDownload() {
    if (RealName) {
        delete RealName;
        RealName=NULL;
    };
    if (NewRealName) {
        delete NewRealName;
        NewRealName=NULL;
    };
    if (HTTP) delete HTTP;
    if (ETag) delete ETag;
    if (Auth) delete Auth;
    if (D_PATH) delete D_PATH;
    if (D_FILE.name) delete (D_FILE.name);
    if (D_FILE.fdesc) close(D_FILE.fdesc);
    if (answer) delete(answer);
    if (RealName) delete RealName;
    if (NewRealName) delete NewRealName;
};
