/*	WebDownloader for X-Window
 *	Copyright (C) 1999 Koshelev Maxim
 *	This Program is free but not GPL!!! You can't modify it
 *	without agreement with autor. You can't distribute modified
 *	program but you can distribute unmodified program.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>

#include "var.h"
#include "ftpd.h"
#include "httpd.h"
#include "hproxy.h"
#include "locstr.h"
#include "main.h"
#include "dlist.h"
#include "mdlist.h"
#include "meter.h"
#include "log.h"
#include "mainlog.h"
#include "signal.h"
#include "savelog.h"
#include "sortstr.h"
#include "face/list.h"
#include "face/buttons.h"
#include "face/log.h"
#include "config.h"

tTwoStrings::tTwoStrings() {
    one=two=NULL;
};

void tTwoStrings::zero() {
    one=two=NULL;
};

tTwoStrings::~tTwoStrings() {};

int get_port_by_proto(char *proto) {
    if (proto) {
        if (equal(proto,"ftp")) return 21;
        if (equal(proto,"http")) return 80;
    };
    return 21;
};

//**********************************************/

tMLog *MainLog=NULL;

tDList *RunList,*StopList,*CompleteList,*PausedList,*WaitStopList;
tDList *WaitList;
tMeter *GlobalMeter=NULL;
tMeter *LocalMeter=NULL;
key_t LogsMsgQueue;

void tMain::init() {
    WaitList=new tDList(DL_WAIT);
    WaitStopList=new tDList(DL_STOPWAIT);
    RunList=new tDList(DL_RUN);
    StopList=new tDList(DL_STOP);
    PausedList=new tDList(DL_PAUSE);
    CompleteList=new tDList(DL_COMPLETE);
    WaitList->init(0);
    WaitStopList->init(0);
    RunList->init(0);
    StopList->init(0);
    PausedList->init(0);
    CompleteList->init(0);
    GlobalMeter=new tMeter;
    GlobalMeter->init(METER_LENGTH);
    LocalMeter=new tMeter;
   	LocalMeter->init(METER_LENGTH);

	LimitsForHosts=new tHostsLimits;
	read_limits();
	SpeedScheduler=new tSpeedQueue;
	SpeedScheduler->init(0);

    LastReadedBytes=0;    
    /* Create msgqueue for logs update */
    LogsMsgQueue=100;
    while ((MsgQueue=msgget(LogsMsgQueue,IPC_CREAT|0600))<0) {
        LogsMsgQueue+=1;
    };
    /* Clearing msgqueue */
    int complete=0;
    while (!complete) {
        mbuf Msg;
        complete=1;
        if (msgrcv(MsgQueue,(msgbuf *)&Msg,sizeof(Msg)-sizeof(long),1,IPC_NOWAIT)>0 ||
                    msgrcv(MsgQueue,(msgbuf *)&Msg,sizeof(Msg)-sizeof(long),2,IPC_NOWAIT)>0)
            complete=0;
    };
};

void tMain::load_defaults() {
    tStringList *list=new tStringList;
    list->init(0);
    read_list(list);
    append_list(list);
    delete(list);
};

void tMain::append_list(tStringList *what){
    tString *temp=what->last();
    while(temp) {
        if (strlen(temp->body)){
            tString *temp2=what->next();
            if (temp2){
            	add_downloading(temp2->body,temp->body);
            	tDownload *download=WaitList->last();
            	download->ScheduleTime=temp2->temp;
            };
        };
        temp=what->next();
    };
};

void tMain::delete_speed_limit(tDownload *what){
	SpeedScheduler->del(what->SpeedLimit);
	delete (what->SpeedLimit);
	what->SpeedLimit=NULL;
};

void tMain::init_main_log(){
	MainLog=new tMLog;
    MainLog->init(CFG.MAX_MAIN_LOG_LENGTH);
    MainLog->init_list(GTK_CLIST(MainLogList));
    MainLog->reinit_file();
    MainLog->add("----------------------------------------",LOG_FROM_SERVER);
    MainLog->add(VERSION_NAME,LOG_WARNING);
    MainLog->add("Normally started",LOG_WARNING);
};

void tMain::redraw_logs() {
    mbuf Msg;
    int complete=0;
    int limit = 0;
    while (!complete && limit<20) {
        complete=1;
        limit++;
        if (msgrcv(MsgQueue,(msgbuf *)&Msg,sizeof(Msg)-sizeof(long),1,IPC_NOWAIT)>0) {
            complete=0;
            log_window_add_string(Msg.which,Msg.what);
        };
        if (msgrcv(MsgQueue,(msgbuf *)&Msg,sizeof(Msg)-sizeof(long),2,IPC_NOWAIT)>0) {
            complete=0;
            del_first_from_log(Msg.which);
        };
    };
};

void tMain::split_string(char *what,char *delim,tTwoStrings *out) {
    char * where=strstr(what,delim);
    if (where) {
        int len=strlen(where),len1=strlen(delim);
        out->two=copy_string(where+len1);
        len1=strlen(what)-len;
        out->one=copy_string(what,len1);
    } else {
        out->two=copy_string(what);
        out->one=NULL;
    };
    delete(what);
};

void tMain::del_completed() {
    tDownload *temp=CompleteList->last();
    while (temp) {
        del_download_from_clist(temp);
        CompleteList->del(temp);
        delete temp;
        temp=CompleteList->last();
    };
};

void tMain::del_fataled() {
    tDownload *temp=StopList->last();
    while (temp) {
        del_download_from_clist(temp);
        StopList->del(temp);
        delete temp;
        temp=StopList->last();
    };
};

void tMain::del_all() {
	if (PausedList->count() || WaitList->count() || RunList->count() ||StopList->count() || CompleteList->count())
		MainLog->add("Clear queue of downloads",LOG_ERROR);
    del_completed();
    del_fataled();
    tDownload *temp=PausedList->last();
    while (temp) {
        del_download_from_clist(temp);
        PausedList->del(temp);
        delete temp;
        temp=PausedList->last();
    };
    temp=WaitList->last();
    while (temp) {
        del_download_from_clist(temp);
        WaitList->del(temp);
        delete temp;
        temp=WaitList->last();
    };
    temp=RunList->last();
    while (temp) {
        stop_thread(temp);
        del_download_from_clist(temp);
        RunList->del(temp);
        delete temp;
        temp=RunList->last();
    };
};

tAddr *tMain::analize(char *what) {
    tTwoStrings pair;
    split_string(what,"://",&pair);
    tAddr *out=new tAddr;
    if (pair.one) {
        out->protocol=pair.one;
    } else {
        out->protocol=copy_string(DEFAULT_PROTO);
    };
    out->host=pair.two;
    if (!out->host) {
        delete(out);
        return NULL;
    };
    split_string(out->host,"/",&pair);
    if (pair.one) {
        out->host=pair.one;
        out->file=pair.two;
    } else {
        out->host=pair.two;
        out->file=pair.one;
    };
    split_string(out->host,"@",&pair);
    out->host=pair.two;
    out->username=pair.one;
    if (out->username) {
        split_string(out->username,":",&pair);
        out->username=pair.one;
        out->pass=pair.two;
    } else {
        out->username=NULL;
        out->pass=NULL;
    };
    if (out->file) {
        char *tmp=parse_percents(out->file);
        if (tmp && equal(out->protocol,"ftp")) {
            delete out->file;
            out->file=tmp;
        };
        if (out->file[strlen(out->file)-1]=='/')
            out->file[strlen(out->file)-1]=0;
        char *prom=rindex(out->file,'/');
        if (prom) {
            out->path=copy_string(prom+1);
            *prom=0;
            prom=out->path;
            out->path=sum_strings("/",out->file);
            delete out->file;
            out->file=prom;
        };
    } else {
        out->file=copy_string("");
    };
    if (!out->path) out->path=copy_string("/");
    split_string(out->host,":",&pair);
    if (pair.one) {
        sscanf(pair.two,"%i",&out->port);
        delete pair.two;
        out->host=pair.one;
    } else {
        out->port=0;
        out->host=pair.two;
    };
    if (equal(out->protocol,"ftp") && index(out->file,'*')){
    		out->mask=1;
    };
    /* Parse # in http urls
     */
    if (equal(out->protocol,"http") && out->file!=NULL) {
        char *tmp=index(out->file,'#');
        if (tmp) {
            *tmp=0;
            tmp=out->file;
            out->file=copy_string(tmp);
            delete(tmp);
        };
    };
    if (out->port==0)
   	    out->port=get_port_by_proto(out->protocol);
    return out;
};

int tMain::run_new_thread(tDownload *what) {
    pthread_attr_t attr_p;
    what->status=READY_TO_RUN;
    if (!what->LOG) {
        what->LOG=new tLog;
        what->LOG->init(CFG.MAX_LOG_LENGTH);
    };
    what->Size.old=0;
    what->delete_editor();
    
    what->SpeedLimit=new tSpeed;
	SpeedScheduler->insert(what->SpeedLimit);

    pthread_attr_init(&attr_p);
    pthread_attr_setdetachstate(&attr_p,PTHREAD_CREATE_JOINABLE);
    pthread_attr_setscope(&attr_p,PTHREAD_SCOPE_SYSTEM);
    return (pthread_create(&what->thread_id,&attr_p,download_last,(void *)what));
};

void tMain::stop_download(tDownload *what) {
	if (WaitStopList->owner(what) && what->action!=ACTION_DELETE){
		what->action=ACTION_STOP;
		return;
	};
    if (RunList->owner(what)){
   		RunList->del(what);
    	tSortString *tmp=LimitsForHosts->find(what->info->host,what->info->port);
    	if (tmp) tmp->curent-=1;
    	char data[MAX_LEN];
    	sprintf(data,"Downloading of file %s from %s was terminated [by user]",what->info->file,what->info->host);
    	MainLog->add(data,LOG_WARNING);
    	if (!stop_thread(what))
	    	WaitStopList->insert(what);
	    else 
	    	PausedList->insert(what);
    	set_pixmap_pause(what->GTKCListRow);
    	what->Status.clear();
    }else{
    	if (WaitList->owner(what)){
	    	WaitList->del(what);
	    	PausedList->insert(what);
    		set_pixmap_pause(what->GTKCListRow);
    		what->Status.clear();
    	};
    };
};

int tMain::delete_download(tDownload *what) {
	if (!what) return 0;
    switch (what->owner) {
        case DL_STOP: {
                StopList->del(what);
                break;
            };
        case DL_COMPLETE: {
                CompleteList->del(what);
                break;
            };
        case DL_RUN: {
                stop_download(what);
            };
        case DL_STOPWAIT:{
				what->action=ACTION_DELETE;
                return 0;
            };
        case DL_PAUSE: {
                PausedList->del(what);
                break;
            };
        case DL_WAIT: WaitList->del(what);
    };
	char data[MAX_LEN];
	sprintf(data,"Delete file %s from queue of downloads",what->info->file);
	MainLog->add(data,LOG_WARNING);
    del_download_from_clist(what);
    delete (what);
   	return 1;
};

void tMain::continue_download(tDownload *what) {
	if (WaitStopList->owner(what) && what->action!=ACTION_DELETE){
		what->action=ACTION_CONTINUE;
		return;
	};
	if (RunList->owner(what)){
		stop_download(what);
		what->action=ACTION_CONTINUE;
		return;
	};
    if (!PausedList->owner(what) && !StopList->owner(what)
    	&& !CompleteList->owner(what)) return;
    if (WaitStopList->owner(what))
        WaitStopList->del(what);
    if (PausedList->owner(what))
        PausedList->del(what);
    if (StopList->owner(what))
        StopList->del(what);
    if (CompleteList->owner(what))
        CompleteList->del(what);
   	tSortString *tmp=LimitsForHosts->find(what->info->host,what->info->port);
   	time_t NOW;
   	time(&NOW);
    if (RunList->count()<CFG.MAX_THREADS && what->ScheduleTime<=NOW && (tmp==NULL || tmp->curent<tmp->upper) && run_new_thread(what)==0) {
   		if (tmp) tmp->curent+=1;
        RunList->insert(what);
        set_pixmap_part_run(what->GTKCListRow);
    } else {
        tDownload *temp=WaitList->last();
        if (!temp || temp->GTKCListRow < what->GTKCListRow)
            WaitList->insert(what);
        else {
        	temp=WaitList->first();
        	while (temp && temp->GTKCListRow < what->GTKCListRow)
        		temp=WaitList->prev();
       		WaitList->insert_before(what,temp);
        };
        set_pixmap_wait(what->GTKCListRow);
    };
    char data[MAX_LEN];
    sprintf(data,"Continue downloading of file %s from %s...",what->info->file,what->info->host);
    MainLog->add(data,LOG_OK);
    what->Attempt.clear();
};

int tMain::complete() {
    return (WaitList->count()+RunList->count())>0?0:1;
};

void tMain::add_dir(tDownload *parent) {
	if (parent->DIR==NULL) return;
	tDownload *temp=parent->DIR->last();
	while(temp){
		parent->DIR->del(temp);
		WaitList->insert(temp);
 		add_download_to_clist(temp);
 		temp=parent->DIR->last();
 	};
};

void tMain::print_info(tDownload *what) {
    char data[MAX_LEN];
    switch(what->finfo.type) {
        case T_FILE: {
                change_clist_data(what->GTKCListRow,FILE_TYPE_COL,"file");
        		int REAL_SIZE=what->finfo.size;
                if (REAL_SIZE==0 && what->who!=NULL)
                	REAL_SIZE=what->who->another_way_get_size();
               	make_number_nice(data,REAL_SIZE);
                change_clist_data(what->GTKCListRow,FULL_SIZE_COL,data);
                if (what->who) what->Size.set(what->who->get_readed());
                if (what->Size.change() || CFG.NICE_DEC_DIGITALS.change()){
                	make_number_nice(data,what->Size.curent);
                	change_clist_data(what->GTKCListRow,DOWNLOADED_SIZE_COL,data);
                	what->Pause=time(NULL);
	               	change_clist_data(what->GTKCListRow,PAUSE_COL,"");
	               	// calculating speed for graph
	    			time_t TimeLeft=time(NULL)-LastTime;
	    			if (what->Size.old==0 && what->who) what->Size.old=what->who->get_start_size();
		    		if (TimeLeft && what->Size.curent>what->Size.old) what->NanoSpeed=(what->Size.curent-what->Size.old)/TimeLeft;
	    			else what->NanoSpeed=0;
                	what->Size.reset();
                }else{
                	what->NanoSpeed=0;
               		if (what->status==DOWNLOAD_GO) {
                   		int Pause=time(NULL)-what->Pause;
                   		if (Pause>=30) {
                       		convert_time(Pause,data);
                       		change_clist_data(what->GTKCListRow,PAUSE_COL,data);
                    	};
                	};
                };
                if (what->Start>0) {
                    convert_time(time(NULL)-what->Start,data);
                    change_clist_data(what->GTKCListRow,TIME_COL,data);
                };
                int temp;
                if (REAL_SIZE)
                    temp=int((float(what->Size.curent)/float(REAL_SIZE))*100);
                else
                    temp=100;
                what->Percent.set(temp);
                sprintf(data,"%i",temp);
                if (what->Percent.change()){
                	change_clist_data(what->GTKCListRow,PERCENT_COL,data);
                	what->Percent.reset();
                };
                /*	Speed calculation
                 */
                int period=int(time(NULL)-what->Start);
                if (period && what->who) {
                    float tmp=float(what->Size.curent-what->who->get_start_size());
                    if (tmp>0) {                    	
                        what->Speed.set(int(tmp/period));
                        if (what->Speed.change()){
                        	sprintf(data,"%i",what->Speed.curent);
                        	change_clist_data(what->GTKCListRow,SPEED_COL,data);
                        	what->Speed.reset();
                        };
                    };
                    /*	Remaining time calculation
                     */
                    if (tmp>0) {
                        tmp=(float(REAL_SIZE-what->Size.curent)*float(period))/tmp;
                        if (tmp>=0 && tmp<24*3660) {
                            convert_time(int(tmp),data);
                            change_clist_data(what->GTKCListRow,ELAPSED_TIME_COL,data);
                        } else
                            change_clist_data(what->GTKCListRow,ELAPSED_TIME_COL,"...");
                    } else
                        change_clist_data(what->GTKCListRow,ELAPSED_TIME_COL,"...");
                };
                break;
            };
        case T_DIR: {
                change_clist_data(what->GTKCListRow,FILE_TYPE_COL,"dir");
                break;
            };
        case T_LINK: {
                change_clist_data(what->GTKCListRow,FILE_TYPE_COL,"link");
                break;
            };
        case T_DEVICE: {
                change_clist_data(what->GTKCListRow,FILE_TYPE_COL,"device");
                break;
            };
        default:
            change_clist_data(what->GTKCListRow,FILE_TYPE_COL,"???");
    };
    if (what->who) {
    	what->Status.set(what->who->get_status());
    	if (what->Status.curent==D_DOWNLOAD && !what->who->reget())
    		what->Status.set(D_DOWNLOAD_BAD);
    	if (what->Status.change()){
    		what->Status.reset();
    		switch (what->Status.curent){
    			case D_QUERYING:{
    				set_pixmap_part_run(what->GTKCListRow);
    				break;
    			};
    			case D_DOWNLOAD:{
    				set_pixmap_run(what->GTKCListRow);
    				break;
    			};
    			case D_DOWNLOAD_BAD:{
    				set_pixmap_run_bad(what->GTKCListRow);
    				break;
    			};
    		};
    	};
    	what->Attempt.set(what->who->treat());
    	if (what->Attempt.change()){
    		what->Attempt.reset();
        	if (what->config.number_of_attempts > 0)
            	sprintf(data,"%i/%i",what->Attempt.curent, what->config.number_of_attempts);
        	else
            	sprintf(data,"%i",what->Attempt.curent);
	        change_clist_data(what->GTKCListRow,TREAT_COL,data);
        };
    };
};


void tMain::redirect(tDownload *what){
	char *newurl;
	newurl=what->who->get_new_url();
	if (newurl){
		RunList->del(what);
       	tDownload *temp=WaitList->first();
   		if (temp)
   			WaitList->insert_before(what,temp);
   		else
   			WaitList->insert(what);
	    tAddr *addr=aa.analize(newurl);
	    if (what->info) delete (what->info);
    	what->info=addr;
	    normalize_path(what->SavePath);
	    what->finfo.type=what->status=0;
	    what->finfo.size=-1;
   		change_clist_data(what->GTKCListRow,URL_COL,newurl);
       	change_clist_data(what->GTKCListRow,FILE_COL,what->info->file);
       	for (int i=FILE_TYPE_COL;i<URL_COL;i++)
            change_clist_data(what->GTKCListRow,i,"");
//	    delete newurl;
		if (what->who) delete what->who;
		what->who=NULL;
	}else{
		what->finfo.type=T_NONE;
	};
};

void tMain::main_circle() {
    freeze_clist();
    tDownload *temp=WaitStopList->last();
    while(temp) {
        tDownload *temp1=WaitStopList->next();
        if (temp->status==DOWNLOAD_REAL_STOP ||
        		temp->status==DOWNLOAD_COMPLETE  ||
        		temp->status==DOWNLOAD_FATAL){
        	real_stop_thread(temp);
			WaitStopList->del(temp);
			PausedList->insert(temp);
			delete_speed_limit(temp);
			if (temp->action==ACTION_DELETE){
				delete_download(temp);
			}else{
				if (temp->action==ACTION_CONTINUE)
					continue_download(temp);
				else
					stop_download(temp);
				temp->action=ACTION_NONE;
			};
		};        	
        temp=temp1;
    };
    temp=RunList->last();
    while(temp) {
        tDownload *temp1=RunList->next();
        print_info(temp);
       	char data[MAX_LEN];
        switch(temp->status) {
            case DOWNLOAD_COMPLETE: {
					delete_speed_limit(temp);
                    RunList->del(temp);
                    tSortString *tmp=LimitsForHosts->find(temp->info->host,temp->info->port);
                    if (tmp) tmp->curent-=1;
                    if (temp->who->file_type()==T_DIR) {
                    	sprintf(data,"Downloading of directory %s://%s%s/%s was completed",temp->info->protocol,temp->info->host,temp->info->path,temp->info->file);
                        add_dir(temp);
                    } else{
                    	int bytes = temp->finfo.size==0 ? temp->who->get_readed():temp->finfo.size;
                    	sprintf(data,"Downloading of file %s://%s%s/%s (%i bytes) was completed at speed %i bytes/sec",
                    				temp->info->protocol,temp->info->host,temp->info->path,temp->info->file,bytes,temp->Speed.curent);
                    	make_file_visible(temp);
                    };
                   	MainLog->add(data,LOG_OK);
                    if (temp->finfo.type==T_REDIRECT) {
                    	redirect(temp);
                    }else{
	                    if (CFG.DELETE_COMPLETED ) {
    	                    del_download_from_clist(temp);
        	                delete (temp);
            	        } else {
                	        if (temp->who) delete(temp->who);
                    	    temp->who=NULL;
                        	CompleteList->insert(temp);
	                        set_pixmap_complete(temp->GTKCListRow);
    	                };
    	            };
                    break;
                };
            case DOWNLOAD_FATAL: {
					delete_speed_limit(temp);
            		delete(temp->SpeedLimit);
                   	sprintf(data,"Downloading of file %s://%s%s/%s was terminated by fatal error",temp->info->protocol,temp->info->host,temp->info->path,temp->info->file);
                   	MainLog->add(data,LOG_ERROR);
                    RunList->del(temp);
                    tSortString *tmp=LimitsForHosts->find(temp->info->host,temp->info->port);
                    if (tmp) tmp->curent-=1;
                    if (CFG.DELETE_FATAL) {
                        del_download_from_clist(temp);
                        delete (temp);
                    } else {
                        if (temp->who) delete(temp->who);
                        temp->who=NULL;
                        StopList->insert(temp);
                        set_pixmap_stop(temp->GTKCListRow);
                    };
                };
        };
        temp=temp1;
    };
    temp=WaitList->first();
    time_t NOW;
    time(&NOW);
    while(temp && RunList->count()<CFG.MAX_THREADS) {
    	tSortString *tmp=LimitsForHosts->find(temp->info->host,temp->info->port);
    	if (temp->ScheduleTime<=NOW && (tmp==NULL || tmp->curent<tmp->upper)){
    		if (tmp) tmp->curent+=1;
        	if (run_new_thread(temp)) 
        		break;
        	WaitList->del(temp);
        	RunList->insert(temp);
        	set_pixmap_part_run(temp->GTKCListRow);
        	temp=WaitList->first();
        }else
        	temp=WaitList->prev();
    };
    unfreeze_clist();
    speed();
    prepare_buttons();
   	CFG.NICE_DEC_DIGITALS.reset();
};

void tMain::make_file_visible(tDownload *what) {
    if (what->finfo.type==T_FILE) {
        char *savepath=what->SavePath;
        char *oldname=sum_strings(savepath,"/.",what->who->get_real_name());
        char *newname=sum_strings(savepath,"/",what->who->get_real_name());
        rename(oldname,newname);
        delete oldname;
        delete newname;
    };
};

//**********************************************/

int tMain::add_downloading(char *adr,char *where) {
    char *temp=copy_string(adr);
    tAddr *addr=analize(temp);
    if (!addr) return -1;
    tDownload *whatadd=new tDownload;
    whatadd->info=addr;
    if (where!=NULL && strlen(where)>0){
        whatadd->SavePath=copy_string(where);
    }else
        whatadd->SavePath=copy_string(CFG.GLOBAL_SAVE_PATH);
    if (strlen(addr->file)==0) {
        whatadd->finfo.type=T_DIR;
        whatadd->finfo.size=0;
    };
    
    normalize_path(whatadd->SavePath);
	
    whatadd->config.timeout=CFG.TIME_OUT;
    whatadd->config.time_for_sleep=CFG.RETRY_TIME_OUT;
    whatadd->config.number_of_attempts=CFG.MAX_RETRIES;
    whatadd->config.passive=CFG.FTP_PASSIVE_MODE;
    whatadd->config.permisions=CFG.FTP_PERMISIONS;
    whatadd->config.get_date=CFG.GET_DATE;
	whatadd->config.proxy_type=CFG.FTP_PROXY_TYPE;


    if (equal(whatadd->info->protocol,"ftp")){
    	if (CFG.USE_PROXY_FOR_FTP){
    		whatadd->config.proxy_host=copy_string(CFG.FTP_PROXY_HOST);
    		whatadd->config.proxy_port=CFG.FTP_PROXY_PORT;
    		if (CFG.NEED_PASS_FTP_PROXY){
	    		whatadd->config.proxy_user=copy_string(CFG.FTP_PROXY_USER);
	    		whatadd->config.proxy_pass=copy_string(CFG.FTP_PROXY_PASS);
    		};
    	};
    }else{
    	if (CFG.USE_PROXY_FOR_HTTP){
    		whatadd->config.proxy_host=copy_string(CFG.HTTP_PROXY_HOST);
    		whatadd->config.proxy_port=CFG.HTTP_PROXY_PORT;
    		if (CFG.NEED_PASS_HTTP_PROXY){
	    		whatadd->config.proxy_user=copy_string(CFG.HTTP_PROXY_USER);
	    		whatadd->config.proxy_pass=copy_string(CFG.HTTP_PROXY_PASS);
    		};
    	};
    };

    WaitList->insert(whatadd);
    add_download_to_clist(whatadd);
    addr=NULL;
    return 0;
};


void tMain::speed() {
    time_t TimeLeft=time(NULL)-LastTime;
    int bytes=READED_BYTES-LastReadedBytes;
    if (TimeLeft>0) {
        int Speed=bytes/TimeLeft;
        LastReadedBytes=READED_BYTES;
        GlobalMeter->add(Speed);
        LastTime=time(NULL);
    };
    int SPEED_LIMIT=-1;
    switch (CFG.SPEED_LIMIT){
    	case 1:{
    		SPEED_LIMIT=CFG.SPEED_LIMIT_1;
    		break;
    	};
    	case 2:{
    		SPEED_LIMIT=CFG.SPEED_LIMIT_2;
    		break;
    	};
    	case 3:
    	default:{
    		SPEED_LIMIT=-1;
    		break;
    	};
    };
    if (SPEED_LIMIT-bytes>0)
    	SPEED_LIMIT+=SPEED_LIMIT-bytes;
	SpeedScheduler->schedule(SPEED_LIMIT);
};

void tMain::run() {
    LastTime=time(NULL);
    gtk_main();
};

void tMain::add_download_message(tDownload *what){
	if (!what) return;
	char data[MAX_LEN];
	sprintf(data,"Added downloading of file %s from %s [by user]",what->info->file,what->info->host);
	MainLog->add(data,LOG_OK);
};

void tMain::done() {
    /* There are  we MUST stop all threads!!!
     */
    tDownload *tmp=RunList->last();
    while (tmp) {
        stop_download(tmp);
        tmp=RunList->next();
    };
    delete(WaitList);
    delete(StopList);
    delete(CompleteList);
    delete(RunList);
    delete(GlobalMeter);
    delete(LocalMeter);
    msgctl(MsgQueue,IPC_RMID,NULL);
};

void tMain::reinit_main_log(){
	MainLog->reinit(CFG.MAX_MAIN_LOG_LENGTH);
	MainLog->reinit_file();
};
//*****************************************************************/
void download_http(tDownload *what) {
    if (!what->who) what->who=new tHttpDownload;
    tAddr *addr=what->info;
    if (what->who->init(addr,what->LOG,&(what->config))) {
        what->status=DOWNLOAD_FATAL;
        what->who->done();
        pthread_exit(NULL);
        return;
    };
    what->who->init_download(addr->path,addr->file);
    /* We need to know size of already loaded file
     * but I think if file not found we need to delete it
     * because in http name of file may be specify 
     * in http answer
     */
    int CurentSize=what->who->create_file(what->SavePath);
    if (CurentSize<0) {
        what->status=DOWNLOAD_FATAL;
        what->who->done();
        pthread_exit(NULL);
        return;
    };
    what->who->set_data(CurentSize);
    int size=what->who->get_size();
    /* In the case if file already loaded
     */
    if (size==CurentSize && size!=0) {
        what->finfo.size=size;
        what->finfo.type=T_FILE;
        what->who->done();
        what->status=DOWNLOAD_COMPLETE;
        pthread_exit(NULL);
        return;
    };
    /* There are must be procedure of removing file
     * wich execute if CurentSize==0
     */
    if (CurentSize==0) {
        if (what->who->delete_file(what->SavePath))
            what->LOG->add("It is strange that we can't delete file which just created...",LOG_WARNING);
    };

    if (size<-1) {
        what->LOG->add("File not found\n",LOG_WARNING);
        what->who->done();
        what->status=DOWNLOAD_FATAL;
        pthread_exit(NULL);
        return;
    };
    if (size==-1) {
        what->finfo.type=T_REDIRECT;
        what->who->done();
        what->status=DOWNLOAD_COMPLETE;
        pthread_exit(NULL);
        return;
    };
    what->finfo.size=size;
    what->finfo.type=T_FILE;
    what->Start=what->Pause=time(NULL);
    /* there we need to create file again
     * if CurentSize==0
     */
    if (CurentSize==0) CurentSize=what->who->create_file(what->SavePath);
    what->status=DOWNLOAD_GO;
    if (what->who->download(CurentSize,what->finfo.size-CurentSize)) {
        what->who->done();
        what->status=DOWNLOAD_FATAL;
        pthread_exit(NULL);
        return;
    };
	what->who->set_date_file(what->SavePath);
    what->who->done();
    what->status=DOWNLOAD_COMPLETE;
};

//-----------------------------------------------------------------

void download_proxy(tDownload *what) {
    if (!what->who) what->who=new tProxyDownload;
    tAddr *addr=what->info;
    if (what->who->init(addr,what->LOG,&(what->config))) {
        what->status=DOWNLOAD_FATAL;
        what->who->done();
        pthread_exit(NULL);
        return;
    };
    what->who->init_download(addr->path,addr->file);
    /* We need to know size of already loaded file
     * but I think if file not found we need to delete it
     * because in http name of file may be specify 
     * in http answer
     */
    int CurentSize=what->who->create_file(what->SavePath);
    if (CurentSize<0) {
        what->status=DOWNLOAD_FATAL;
        what->who->done();
        pthread_exit(NULL);
        return;
    };
    what->who->set_data(CurentSize);
    int size=what->who->get_size();
    /* In the case if file already loaded
     */
    if (size==CurentSize && size!=0) {
        what->finfo.size=size;
        what->finfo.type=T_FILE;
        what->who->done();
        what->status=DOWNLOAD_COMPLETE;
        pthread_exit(NULL);
        return;
    };
    /* There are must be procedure of removing file
     * wich execute if CurentSize==0
     */
    if (CurentSize==0) {
        if (what->who->delete_file(what->SavePath))
            what->LOG->add("It is strange that we can't delete file which just created...",LOG_WARNING);
    };

    if (size<-1) {
        what->LOG->add("File not found",LOG_ERROR);
        what->who->done();
        what->status=DOWNLOAD_FATAL;
        pthread_exit(NULL);
        return;
    };
    if (size==-1) {
        what->finfo.type=T_REDIRECT;
        what->who->done();
        what->LOG->add("We can redirect from proxy!",LOG_ERROR);
        what->status=DOWNLOAD_FATAL;
        pthread_exit(NULL);
        return;
    };
    what->finfo.size=size;
    what->finfo.type=T_FILE;
    what->Start=what->Pause=time(NULL);
    /* there we need to create file again
     * if CurentSize==0
     */
    if (CurentSize==0) CurentSize=what->who->create_file(what->SavePath);
    what->status=DOWNLOAD_GO;
    if (what->who->download(CurentSize,what->finfo.size-CurentSize)) {
        what->who->done();
        what->status=DOWNLOAD_FATAL;
        pthread_exit(NULL);
        return;
    };
    what->who->set_date_file(what->SavePath);
    what->who->done();
    what->status=DOWNLOAD_COMPLETE;
};


//-----------------------------------------------------------------
void *download_last(void *nothing) {
    tDownload *what=(tDownload *)nothing;
	my_pthread_key_init();
	*(my_pthread_key_get())=what;
    init_signal_handler();
    if (what) {
        tAddr *addr=what->info;
        what->LOG->MsgQueue=msgget(LogsMsgQueue,0);
        if (what->LOG->MsgQueue<0) {
            what->LOG->add("Can't open messages queue!",LOG_ERROR);
            what->status=DOWNLOAD_FATAL;
            pthread_exit(NULL);
            return NULL;
        };
      	if (what->config.proxy_host && what->config.proxy_type){
       		download_proxy(what);
            pthread_exit(NULL);
   	        return NULL;
       	};
        if (addr==NULL || equal_first(addr->protocol,"ftp")) {
            if (!what->who) what->who=new tFtpDownload;
        } else {
            download_http(what);
            pthread_exit(NULL);
            return NULL;
        };
        what->LOG->add("Was Started!\n",LOG_WARNING);

        if (what->finfo.type==T_LINK) {
            what->LOG->add("It is a link and we already load it\n",LOG_WARNING);
            what->who->short_init(what->LOG);
            what->who->init_download(addr->path,addr->file);
            what->who->set_file_info(&(what->finfo));
            what->who->create_file(what->SavePath);
            what->who->set_date_file(what->SavePath);
   	        what->who->done();
	        what->status=DOWNLOAD_COMPLETE;
            pthread_exit(NULL);
            return NULL;
        };

        if (what->who->init(addr,what->LOG,&(what->config))) {
	        what->who->done();
            what->status=DOWNLOAD_FATAL;
            pthread_exit(NULL);
            return NULL;
        };

        what->who->init_download(addr->path,addr->file);
        if (what->finfo.size<0) {
            what->status=DOWNLOAD_SIZE_WAIT;
            int size=what->who->get_size();
            if (size<0) {
                what->LOG->add("File not found\n",LOG_ERROR);
		        what->who->done();
                what->status=DOWNLOAD_FATAL;
                pthread_exit(NULL);
                return NULL;
            };
            what->finfo.size=size;
            what->finfo.type=what->who->file_type();
        } else {
            what->who->set_file_info(&(what->finfo));
        };
        int CurentSize=0;
        if (what->info->mask==0)
        	CurentSize=what->who->create_file(what->SavePath);

        if (what->finfo.type==T_DEVICE){
	        what->who->done();
	        what->status=DOWNLOAD_COMPLETE;
            pthread_exit(NULL);
            return NULL;
        };
        	
        if (CurentSize<0) {
	        what->who->done();
            what->status=DOWNLOAD_FATAL;
            pthread_exit(NULL);
            return NULL;
        };

        what->status=DOWNLOAD_GO;
        what->Start=what->Pause=time(NULL);
        if (what->who->download(CurentSize,what->finfo.size-CurentSize)) {
	        what->who->done();
            what->status=DOWNLOAD_FATAL;
            pthread_exit(NULL);
            return NULL;
        };
        what->who->set_date_file(what->SavePath);
        what->who->done();
        if (what->finfo.type==T_DIR) what->convert_list_to_dir();
        what->status=DOWNLOAD_COMPLETE;
    };
    pthread_exit(NULL);
    return NULL;
};
//------------------------------------------------------------------------

